#!/usr/bin/perl
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;

use strict;
use warnings;
use Test::More;
use Test::Exception;

lives_ok {
    package My::Class;
    use Mouse;

    has s_rw => (
        is => 'rw',
    );

    has s_ro => (
        is => 'ro',
    );

    has a_rw => (
        is  => 'rw',
        isa => 'ArrayRef',

        auto_deref => 1,
    );

    has a_ro => (
        is  => 'ro',
        isa => 'ArrayRef',

        auto_deref => 1,
    );

    has h_rw => (
        is  => 'rw',
        isa => 'HashRef',

        auto_deref => 1,
    );

    has h_ro => (
        is  => 'ro',
        isa => 'HashRef',

        auto_deref => 1,
    );
} 'class definition';

lives_ok {
    my $o = My::Class->new();

    is_deeply [scalar $o->s_rw], [undef], 'uninitialized scalar attribute/rw in scalar context';
    is_deeply [$o->s_rw],        [undef], 'uninitialized scalar attribute/rw in list context';
    is_deeply [scalar $o->s_ro], [undef], 'uninitialized scalar attribute/ro in scalar context';
    is_deeply [$o->s_ro],        [undef], 'uninitialized scalar attribute/ro in list context';


    is_deeply [scalar $o->a_rw], [undef], 'uninitialized ArrayRef attribute/rw in scalar context';
    is_deeply [$o->a_rw],        [],      'uninitialized ArrayRef attribute/rw in list context';
    is_deeply [scalar $o->a_ro], [undef], 'uninitialized ArrayRef attribute/ro in scalar context';
    is_deeply [$o->a_ro],        [],      'uninitialized ArrayRef attribute/ro in list context';

    is_deeply [scalar $o->h_rw], [undef], 'uninitialized HashRef attribute/rw in scalar context';
    is_deeply [$o->h_rw],        [],      'uninitialized HashRef attribute/rw in list context';
    is_deeply [scalar $o->h_ro], [undef], 'uninitialized HashRef attribute/ro in scalar context';
    is_deeply [$o->h_ro],        [],      'uninitialized HashRef attribute/ro in list context';

} 'testing';

done_testing;
