#!/usr/bin/perl
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;

use strict;
use warnings;

use lib 't/lib', 'lib';

use Test::More;
use Test::Exception;


$SIG{__WARN__} = sub { 0 };

eval { require Foo; };
ok(!$@, '... loaded Foo successfully') || diag $@;

delete $INC{'Foo.pm'};

eval { require Foo; };
ok(!$@, '... re-loaded Foo successfully') || diag $@;

eval { require Bar; };
ok(!$@, '... loaded Bar successfully') || diag $@;

delete $INC{'Bar.pm'};

eval { require Bar; };
ok(!$@, '... re-loaded Bar successfully') || diag $@;

done_testing;
