use strict;
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use t::lib::MooseCompat;
use warnings;

use Test::More;
use Test::Exception;

use Mouse::Meta::Class;

{
    my $meta;
    lives_ok {
        $meta = Mouse::Meta::Class->create_anon_class(
            superclasses => [ 'Mouse::Object', ],
        );
    }
    'Class is created successfully';
}

done_testing;
